package owltools.gaf.closure;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.lang3.tuple.Pair;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectInverseOf;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLObjectPropertyExpression;
import org.semanticweb.owlapi.model.OWLPropertyExpressionVisitor;

import owltools.graph.OWLGraphWrapper;

public class ClosureTool {
	
	private final OWLGraphWrapper graph;
	private final Map<String, Pair<Set<OWLObjectProperty>,List<String>>> propertySets;

	public ClosureTool(OWLGraphWrapper graph, Map<String, Set<OWLObjectProperty>> propertySets) {
		this.graph = graph;
		this.propertySets = new HashMap<String, Pair<Set<OWLObjectProperty>,List<String>>>();
		for(Entry<String, Set<OWLObjectProperty>> propSet : propertySets.entrySet()) {
			final Set<OWLObjectProperty> descendantsClosure = new HashSet<OWLObjectProperty>();
			for(OWLObjectProperty p : propSet.getValue()) {
				if (descendantsClosure.contains(p)) {
					continue;
				}
				Set<OWLObjectPropertyExpression> set = graph.getSubPropertyReflexiveClosureOf(p);
				for (OWLObjectPropertyExpression expr : set) {
					expr.accept(new OWLPropertyExpressionVisitor() {
						
						@Override
						public void visit(OWLDataProperty property) {
							// ignore
						}
						
						@Override
						public void visit(OWLObjectInverseOf property) {
							// ignore
						}
						
						@Override
						public void visit(OWLObjectProperty property) {
							descendantsClosure.add(property);
						}
					});
				}
			}
		}
	}

	public Map<String, Map<String, String>> findClosures(OWLClass directAnnotation, Set<OWLClassExpression> expressions) {
		Map<String, Map<String, String>> closures = new HashMap<String, Map<String, String>>();
		for(Entry<String, Pair<Set<OWLObjectProperty>,List<String>>> propSet : propertySets.entrySet()) {
			final String propSetName = propSet.getKey();
			final Pair<Set<OWLObjectProperty>, List<String>> pair = propSet.getValue();
			
			//graph.getAncestors(directAnnotation, relevantProperties);
			Map<String, String> closure = graph.getRelationClosureMap(directAnnotation, pair.getRight());
		}
		return closures;
	}
}
